/*
 *  function joins two Huffman code trees to create a third
 *
 *   arguments:  pht1 - pointer to first tree
 *               pht2 - pointer to second tree
 *               status - pointer to object of type BOOLEAN
 *
 *   effects:
 *       creates new tree, with 
 *          root pointing to pht1 and pht2, 
 *          null name at root node
 *          frequency count at root node equal to pht1 ct + pht2 ct
 *
 *   values returned:
 *       -.  on failure,  status flag in object set to FALSE
 *       -.  on success,  status flag in object set to TRUE,
 *              and tree resulting from join returned;
 *              roots of pht1, pht2 become left, right subtrees
 * 
 */

#include <stdio.h>

#include "httypes.h"

#include "fifns.h"
#include "fcfns.h"

#include "htnfns.h"
#include "htfns.h"

pHTree htjoin ( pHTree pht1, pHTree pht2, BOOLEAN *status )
{ 
  pHTree pht;
  pHTreeNode phtn;

  /* 
   * first, make an empty tree whose count, at the root,
   * is the tally of its prospective left and right subtrees
   */
  pht = htmake(status);
  if (! (*status)) return nilHT;

  /* 
   * then, join the two supplied trees into a new subtree
   */
  phtn = htnjoin(getHTRoot(pht1), getHTRoot(pht2), status);
  if (! (*status)) {
      htdestroy(pht);
      return nilHT;
  }

  /* finally, reset root of new tree to be root of the joined subtree */
  putHTRoot(pht, phtn);

  return pht;
} 
