/*
 *  function joins two Huffman code subtrees to create a third
 *
 *   arguments:  pht1 - pointer to first tree
 *               pht2 - pointer to second tree
 *               status - pointer to object of type BOOLEAN
 *
 *   effects:
 *       creates new node, with 
 *          root pointing to pht1 and pht2, 
 *          null name at root node
 *          frequency count at root node equal to pht1 ct + pht2 ct
 *
 *   values returned:
 *       -.  on failure,  status flag in object set to FALSE
 *       -.  on success,  status flag in object set to TRUE,
 *              and tree resulting from join returned;
 *              roots of pht1, pht2 become left, right subtrees
 * 
 */

#include <stdio.h>

#include "htntypes.h"

#include "fifns.h"
#include "fcfns.h"

#include "htnfns.h"

pHTreeNode htnjoin ( pHTreeNode pht1, pHTreeNode pht2, BOOLEAN *status )
{ 
  pHTreeNode phtn;
  unsigned newfreqCt = getHTNFreqCt(pht1) + getHTNFreqCt(pht2);

  /* 
   * first, make an empty tree node whose count, at the root,
   * is the tally of its prospective left and right subtrees
   */

  phtn = htnmake(fimake(' ',FALSE),fcmake(newfreqCt),status);
  if (! (*status)) return nilHTNode;

  /* then, finish consing the left, right subtrees */
  putHTNLeft(phtn,pht1);
  putHTNRite(phtn,pht2);

  return phtn;
} 
