/*
 *  function allocates and initializes a node in a Huffman code tree
 *
 *   three arguments:
 *      pni     - pointer to an item-name object for frequency count
 *      pfc     - pointer to a frequency count object
 *      status  - BOOLEAN status flag
 *
 *   value returned:  
 *      -.  on failure, status flag is FALSE
 *      -.  on success, status flag is TRUE, 
 *          routine returns pointer to a Huffman code tree node object
 *          whose left and right children are initialized to nil
 */

#include <stdio.h>

#include "htntypes.h"

#include "fcfns.h"
#include "fifns.h"
#include "htnfns.h"

pHTreeNode htnmake (pFItemname pni, pFCount pfc, BOOLEAN *status)
{  
   pHTreeNode phtn  = (pHTreeNode) malloc(sizeof(HTreeNode));

   if (phtn == 0) {
      fprintf(stderr,"?? out of space failure in htinit\n");
      fprintf(stderr,"?? cannot make tree element\n");
      *status = FALSE;
   } else {
      putHTNpOccCount(phtn,pfc);
      putHTNpItemName(phtn,pni);

      putHTNLeft(phtn,nilHTNode);
      putHTNRite(phtn,nilHTNode);

      *status = TRUE;
   }
   return phtn;
}
