/*
 *  dump one element in a tree [pointed to by first arg]
 *     to standard output, centered in a caller-specified no. of spaces.
 *     
 *  return true iff number of spaces allotted for dump --
 *     second argument -- suffices to dump element.
 */
    
#include <stdio.h>

#include "htntypes.h"

#include "fifns.h"
#include "fcfns.h"

#include "htnfns.h"


BOOLEAN htnodedump (pHTreeNode pt, unsigned fldwidth)
{
   /* 
    *  next two calls allocate print strings for node name, freq count
    */
   char *nodename = fidump(getHTNpItemName(pt));   
   char *freqcnt  = fcdump(getHTNpOccCount(pt));

   unsigned nodenamelen, freqcntlen, totalgap, leftgap, rightgap;
   
   nodenamelen = strlen(nodename);
   freqcntlen = strlen(freqcnt);

   /*
    * check and see if there's enough space in string to form item description
    * ask for min one space on either side of node data, + room for ":"  
    */
   if (nodenamelen + freqcntlen + 3 > fldwidth) {
      free(nodename);
      free(freqcnt);
      return FALSE;   /* give up:  can't fit data into allocated space */
   }

   /*  set aside space on left, right side of print string */
   totalgap = fldwidth - (nodenamelen + freqcntlen + 1);
   rightgap = totalgap / 2;     /* truncating divide ensures that           */
   leftgap = (totalgap+1) / 2;  /* odd space, if any, will be left of field */
 
   /*  print item descriptor to standard output */
   printf("%*c%s%c%s%*c",leftgap,' ',nodename,':',freqcnt,rightgap,' ');

   /*  and deallocate the intermediate descriptors  */
   free(nodename);
   free(freqcnt);

   return TRUE;
}
