/*  function for setting to a position in a tree.
 *
 *  arguments:
 *     pHTree -  a pointer to the head of a Huffman tree
 *     locstr -  a string that describes a location in a tree, as follows:
 *         -.  locstr can consists of any number of l's and r's
 *         -.  the empty string sets to the tree's root node
 *         -.  the effect of invoking the routine with a string 
 *             of the form "x..xxxxr" is the same as
 *             -.   the effect of first invoking the routine with 
 *                  the string's prefix "x..xxxx", then
 *             -.   setting to the right child of the node selected
 *                  by the operation above
 *         -.  the effect of invoking the routine with a string 
 *             of the form "x..xxxxl" is the same as
 *             -.   the effect of first invoking the routine with 
 *                  the string's prefix "x..xxxl", then
 *             -.   setting to the left child of the node selected
 *                  by the operation above
 *     status -  a pointer to a value of type BOOLEAN
 *  
 *   values returned:
 *     on failure, status is FALSE.  
 *        (string was ill-formed, or specified node was not in tree)
 *     on success, status is TRUE, 
 *        and the routine returns a pointer to the tree node selected.
 *
 *   effects:
 *      error message printed to stderr if selector string invalid,
 *      or string names node not in tree
 */

#include <stdio.h>

#include "httypes.h"

#include "htnfns.h"
#include "htfns.h"

pHTreeNode htsetposn  (pHTree pt, char *position, BOOLEAN *status)
{
   pHTreeNode cur_node = getHTRoot(pt);
   char *cur_selector = position;

   while  ((*cur_selector) != '\0') {
       switch (*cur_selector) {
         case 'l':
         case 'L':  
             if (isNilHTNode(cur_node)) {
                fprintf(stderr,
                   "?? htsetposn: selector string %s names non-tree node\n\n",
                   position); 
                *status = FALSE;
                return nilHTNode;
             }
             cur_node = getHTNLeft(cur_node);
             break;
         case 'r':
         case 'R':
             if (isNilHTNode(cur_node)) {
                fprintf(stderr,
                   "?? htsetposn: selector string %s names non-tree node\n\n",
                   position); 
                *status = FALSE;
                return nilHTNode;
             }
             cur_node = getHTNRite(cur_node);
             break;
         default:
             fprintf(stderr,
                   "?? htsetposn: selector string %s has invalid char: %c/n/n",
                   position, 
                   *cur_selector);
             *status = FALSE;
             return nilHTNode;
       }
       cur_selector++;
   }
 
   *status=TRUE;
   return cur_node;
}
