/*
 *   Interactive subroutine for browsing a tree on the terminal
 *
 *   Inputs:
 *       pHTree -  Huffman code tree to browse
 *
 *   Outputs:  none
 *
 *   Routine repeatedly asks user for requisite options,
 *      then invokes displaytree to display desired tree fragment.
 */

#include <stdio.h>

#include "httypes.h"

#include "htnfns.h"
#include "htfns.h"

/* change the next definition for wider/narrower displays */
#define SCREEN_WID 80
static unsigned screen_width = SCREEN_WID;

/*
 *   support procedures
 *      -.  get option from user
 *      -.  set current position in tree, relative to root
 */  

   static unsigned getoption( )
   {
     unsigned option;

     /* break statement exits this loop */
     do {
        char c;
        printf("Tree display routine options include:  \n\n");
        printf("  0 = Exit routine \n");
        printf("  1 = Set width of screen \n");
        printf("  2 = Set start node for display, relative to current node \n");
        printf("  3 = Reset display start node to tree root \n");
        printf("  4 = Display tree \n\n");
        printf(" Enter option: ");
        fflush(stdout);                    /* flush message to std output */

        scanf("%u",&option);               /* read response, then */
        while ((c = getchar()) != '\n');   /* discard to EOL      */

        printf("\n");

        if ((0 <= option) && (option <= 4)) break;
    
        printf("?? unknown option -- reenter\n\n");

      } while (TRUE);
      return option;
   }

   static pHTreeNode setposn(pHTreeNode phtn)
   {
    char posnstring[11], c;
    unsigned i;
    BOOLEAN status;
    pHTreeNode new_phtn;

    printf("enter string specifying new head node, relative to current\n");
    printf("-.  l's move down and left\n");
    printf("-.  r's move down and right\n");
    printf("e.g., lr selects right child of left child of current root\n");
    printf("enter up to ten characters, followed by <ret>: ");
    fflush(stdout);                    /* flush message to standard output */

    i = 0;
    while ((c = getchar()) != '\n') {   /* accept, or discard, to EOL  */
       if (i < 10) {
           posnstring[i] = c;
           i++;
       }
    }
    posnstring[i] = '\0';   /* null-terminate string */

    /*  treat trees, tree nodes as interchangeable */
    new_phtn = htsetposn((pHTree) phtn, posnstring, &status);

    /*  return new if operation succeeded, else old  */
    return (status ? new_phtn : phtn);
}

void htshowtree(pHTree pt)
{
   /* take posn of top node as initial current */
   pHTreeNode cur_node = getHTRoot(pt);  
   char c;

   printf("Huffman tree display routine\n\n");

   /*  loop exits with return statement */
   do {
       switch (getoption()) {
           case 0:  return;
 
           /* enter dialogue to reset screen width */
           case 1:  printf("current screenwidth is %d columns\n",screen_width);
		    printf("enter new screen width: ");
                    fflush(stdout);   /* flush message to standard output */

		    scanf("%u",&screen_width);        /* read response, then */
                    while ((c = getchar()) != '\n');  /* discard to EOL      */
		    break;

           /* enter dialogue to reset node position to lower in tree */
           case 2:  cur_node = setposn(cur_node);
                    break;

           /* reset node position to top of tree */
           case 3:  cur_node = getHTRoot(pt);
                    break;

           /* print tree; treat tree nodes and trees as isomorphic structures */
           case 4:  httreedump((pHTree) cur_node, screen_width);
                    break;

      }
  } while (TRUE);
}
