/*
 *  function deallocates a priority queue
 *
 *    one argument:  q   - a pointer to a priority queue to deallocate
 *
 *  note:  function not only deallocates queue, but all items pointed to by
 *         slots in queue.  This would be a potential source of error
 *         if items in queue were shared at time of function's invocation.
 */

#include <stdio.h>

#include "pqtypes.h"
#include "pqfns.h"

void pqdestroy ( pPriQ q ) 
{  
   if (! isNilPriQ(q)) {

     /*
      *  first, destroy dynamically allocated objects in the queue;
      *  i.e., repeatedly remove and free any code trees in the queue
      */
      while (! isEmptyPriQ(q)) {
         free(getPQElt(getPriQelt(q,1)));
         if (! pqrem(q)) {
             fprintf(stderr,"?? programmer error in pqdestroy");
             fprintf(stderr,"?? couldn't remove item from nonempty queue");
             exit(-1);
         }
       }

      /*
       * then free the queue's data area, 
       *      which must always have at least one element 
       *
       */
      free(getPriQdata(q));   

      /*
       * then free the queue's bookkeeping storage
       */
      free(q);                
   }
}
