/*  priority queue function definitions file */

#ifndef PQFNS_HDR
#define PQFNS_HDR

#ifndef PQTYPES_HDR
#include  "pqtypes.h"
#endif  /* PQTYPES_HDR */

/* 
 *   access macros for manipulating individual elements of priority queues
 *   putPQEltPri() not needed when using pri queues to build Huffman trees -- 
 *   -.  node priorities assigned at node creation don't change
 *
 */
#define getPQElt(qe)            ((qe)->huffTree)
#define getPQEltPri(qe)         (getHTNFreqCt(getPQElt(qe)))

#define putPQElt(qe,val)        ((qe)->huffTree = (val))

#define lteqPQElts(qe1,qe2)    (getPQEltPri(qe1) <= getPQEltPri(qe2))

/*  
 *   access macros for manipulating priority queues proper
 */
#define isNilPriQ(q)          ((q) == nilPriQ)

#define getPriQdata(q)        ((q)->ptrQ)
#define getPriQcap(q)         ((q)->capacity)
#define getPriQffree(q)       ((q)->ffree)

#define putPriQdata(q,val)    ((q)->ptrQ  = (val))
#define putPriQcap(q,val)     ((q)->capacity = (val))
#define putPriQffree(q,val)   ((q)->ffree = (val))

#define initPriQffree(q)      ((q)->ffree = 1)

/*  access macros for manipulating priority queue elements by index  */
#define getPriQminidx(q)      (1)
#define getPriQmaxidx(q)      (getPriQffree(q)-1)
#define isPriQidx(q,ix)       (((ix) <= getPriQmaxidx(q)) && (ix) >= getPriQminidx(q))

#define getPriQelt(q,ix)      (((q)->ptrQ)[(ix)])
#define putPriQelt(q,ix,val)  (((q)->ptrQ)[(ix)] = (val))

#define isEmptyPriQ(q)        (getPriQffree(q) == getPriQminidx(q))
/*
 * test checks if index of first free elt is 1 over capacity because 
 * this priority queue package's indices are 1-based, for simplicity
 */
#define isFullPriQ(q)         (getPriQffree(q) == (getPriQcap(q)+1))


/*   access macros for manipulating priority queue's children/parent
 *   by index
 */

#define getLidx(ix)           (2*(ix))
#define getRidx(ix)           (2*(ix)+1)
#define getParentidx(ix)      ((ix)/2)

#define PriQhasL(q,ix)        (isPriQidx(q,getLidx(ix)))
#define getPriQL(q,ix)        (getPriQelt(q,getLidx(ix)))
#define putPriQL(q,ix,val)    (putPriQelt(q,getLidx(ix),val))
#define PriQhasR(q,ix)        (isPriQidx(q,getRidx(ix)))
#define getPriQR(q,ix)        (getPriQelt(q,getRidx(ix)))
#define putPriQR(q,ix,val)    (putPriQelt(q,getRidx(ix),val))

#define lteqPriQ(q,i1,i2)     ((! isPriQidx(q,i2)) || (! isPriQidx(q,i1)) || lteqPQElts(getPriQelt(q,i1),getPriQelt(q,i2)))

#define swapPriQelt(q,i1,i2)  {pPriQElt t; t=getPriQelt(q,i1); putPriQelt(q,i1,getPriQelt(q,i2)); putPriQelt(q,i2,t);}

/* operators for manipulating priority queues */
extern pPriQ    pqmake     ( unsigned capacity, BOOLEAN *status );
extern void     pqinit     ( pPriQ q );
extern void     pqsiftdn   ( pPriQ q, unsigned idx );
extern void     pqsiftup   ( pPriQ q, unsigned idx );
extern BOOLEAN  pqadd      ( pPriQ q, pHTree pitem );
extern BOOLEAN  pqrem      ( pPriQ q );
extern BOOLEAN  pqputatend ( pPriQ q, pHTree pitem );
extern void     pqeltdump  ( pPriQElt q ); 
extern void     pqdump     ( pPriQ q );
extern void     pqdestroy  ( pPriQ q );

#endif  /* PQFNS_HDR */
