/*
 *  function allocates and initializes priority queue of a user-specified size.
 *
 *    two argument:    
 *         "capacity", the number of elements in priority queue.
 *         BOOLEAN status flag
 *
 *    value returned:  
 *         on failure, status field set to FALSE.
 *         on success, status field set to TRUE, and 
 *              function returns desired "priority queue"
 *
 *  Notes:
 *
 *  -.  routine actually allocates queue with capacity+1 elements--
 *      zeroth element is left unused, for algorithmic convenience.
 */

#include <stdio.h>

#include "pqtypes.h"
#include "pqfns.h"

pPriQ pqmake ( unsigned capacity, BOOLEAN *status )
{
   ppPriQElt qarray = (ppPriQElt) calloc(capacity+1, sizeof(PriQElt));
   pPriQ     q      = (pPriQ)     malloc(sizeof(PriQ));
   
   if (q == 0) {
      fprintf(stderr,"?? out of space failure in pqmake\n");
      fprintf(stderr,"?? cannot make priority queue\n");
      *status = FALSE;
   } else {
      if (qarray == 0) {
          fprintf(stderr,"?? out of space failure in pqmake\n");
          fprintf(stderr,"?? cannot make priority queue element save area\n");
          *status = FALSE;
      } else {
          putPriQdata(q,qarray);
          putPriQcap(q,capacity);
          initPriQffree(q);
          *status = TRUE;
      }
   }
   return q;
}
